local f = CreateFrame("Frame")

local factions = {
	[0] = "Horde",
	[1] = "Alliance",
}

local colorTable = {
	["Death Knight"] = {
		["r"] = 0.77,
		["g"] = 0.12,
		["b"] = 0.23,
	},
	["Druid"] = {
		["r"] = 1,
		["g"] = 0.49,
		["b"] = 0.04,
	},
	["Hunter"] = {
		["r"] = 0.67,
		["g"] = 0.83,
		["b"] = 0.45,
	},
	["Mage"] = {
		["r"] = 0.41,
		["g"] = 0.8,
		["b"] = 0.94,
	},
	["Paladin"] = {
		["r"] = 0.96,
		["g"] = 0.55,
		["b"] = 0.73,
	},
	["Priest"] = {
		["r"] = 1,
		["g"] = 1,
		["b"] = 1,
	},
	["Rogue"] = {
		["r"] = 1,
		["g"] = 0.96,
		["b"] = 0.41,
	},
	["Shaman"] = {
		["r"] = 0,
		["g"] = 0.44,
		["b"] = 0.87,
	},
	["Warlock"] = {
		["r"] = 0.58,
		["g"] = 0.51,
		["b"] = 0.79,
	},
	["Warrior"] = {
		["r"] = 0.78,
		["g"] = 0.61,
		["b"] = 0.43,
	},
}

local BNFriends = {}

local function ToonInfo(id)
	local _, _, _, toon = BNGetFriendInfoByID(id)

	if BNFriends[id] then
		for k,v in pairs(BNFriends[id]) do
			if v.name == toon then
				return v.name, v.client, v.realm, v.faction, v.race, v.class, v.zone, v.level
			end
		end
	end
end

function BNWhisperCharacter_Filter(self, event, msg, author, ...)
	local presenceID = select(11, ...)
	local colorString

	if ToonInfo(presenceID) then
		local name, client, realm, faction, _, class = ToonInfo(presenceID)	

		if client == "WoW" and realm == GetRealmName() and UnitFactionGroup("player") == faction then
			if colorTable[class] then
				colorString = string.format("\124cff%.2x%.2x%.2x", colorTable[class].r*255, colorTable[class].g*255, colorTable[class].b*255)
				msg = string.format("|Hplayer:%s|h("..colorString.."%s|r)|h %s", name, name, msg)				
			else
				msg = string.format("|Hplayer:%s|h(%s)|h %s", name, name, msg)
			end
		end
	end	

	return false, msg, author, ...
end

local function OnHyperlinkEnter(self, linkData, link)
	if linkData:match("^(.-):") == "BNplayer" then
		local _, _, presenceID = linkData:match("^(.-):(.-):(.-):")

		local name, client, realm, faction, race, class, zone, level = ToonInfo(tonumber(presenceID))

		if name then
			GameTooltip:ClearLines()
			ShowUIPanel(GameTooltip)
			GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
			if realm == GetRealmName() then
				GameTooltip:AddLine(name, colorTable[class].r, colorTable[class].g, colorTable[class].b, 1)
			else
				GameTooltip:AddLine(name.."-"..realm, colorTable[class].r, colorTable[class].g, colorTable[class].b, 1)
			end
			GameTooltip:AddLine(faction, 1, 1, 1, 1)
			GameTooltip:AddLine(level.." "..race.." "..class, 1, 1, 1, 1)
			GameTooltip:AddLine(zone, 1, 1, 1, 1)
			GameTooltip:AddLine(client, 1, 1, 1, 1)
			GameTooltip:Show()
		end
	end
end

local function OnHyperlinkLeave(self, linkData, link)
	if linkData:match("^(.-):") == "BNplayer" then
		HideUIPanel(GameTooltip)
	end
end

local function OnEvent(self, event, ...)
	local arg1 = ...

	if event == "ADDON_LOADED" and arg1 == "BNWhisperCharacter" then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", BNWhisperCharacter_Filter)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_CONVERSATION", BNWhisperCharacter_Filter)

		self:UnregisterEvent("ADDON_LOADED")
	elseif event == "UPDATE_CHAT_WINDOWS" or event == "VARIABLES_LOADED" then
		for i=1, NUM_CHAT_WINDOWS do
			local w = _G["ChatFrame"..i]

			w:HookScript("OnHyperlinkEnter", OnHyperlinkEnter)
			w:HookScript("OnHyperlinkLeave", OnHyperlinkLeave)
		end

		if ChatFrame then
			ChatFrame:HookScript("OnHyperlinkEnter", OnHyperlinkEnter)
			ChatFrame:HookScript("OnHyperlinkLeave", OnHyperlinkLeave)
		end
	end

	if event == "BN_FRIEND_ACCOUNT_ONLINE" or event == "VARIABLES_LOADED" then
		for i=1,BNGetNumFriends() do
			BNFriends[BNGetFriendInfo(i)] = {}

			for t=1,BNGetNumFriendToons(i) do
				local _, name, client, realmName, _, toonFaction, race, class, _, zoneName, level = BNGetFriendToonInfo(i, t)

				BNFriends[BNGetFriendInfo(i)][t] = {
					["name"] = name,
					["client"] = client,
					["realm"] = realmName,
					["faction"] = factions[toonFaction],
					["race"] = race,
					["class"] = class,
					["zone"] = zoneName,
					["level"] = level,
				}
			end
		end
	end
end

f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE")
f:RegisterEvent("UPDATE_CHAT_WINDOWS")
f:RegisterEvent("VARIABLES_LOADED")

f:SetScript("OnEvent", OnEvent)